#include "StdAfx.h"
#include "ChartWrapper.h"

#include "Aspect.h"
#include "BarSeries.h"
#include "LineSeries.h"
#include "Brush.h"
#include "Canvas.h"
#include "Marks.h"
#include "Pen.h"
#include "PointSeries.h"
#include "Pointer.h"
#include "Scroll.h"
#include "Series.h"
#include "TeeFunction.h"
#include "Zoom.h"
#include "tchart.h"
#include "axes.h"
#include "axis.h"
#include "axislabels.h"
#include "axistitle.h"
#include "titles.h"
#include "legend.h"
#include "export.h"
#include "toollist.h"
#include "tools.h"
#include "rotatetool.h"
#include "printer.h"
#include "bmpexport.h"
#include "pngexport.h"
#include "jpegexport.h"
#include "horizbarseries.h"
#include "panel.h"
#include "gradient.h"
#include "walls.h"
#include "wall.h"
#include "teefont.h"
#include "bezierseries.h"
#include "strings.h"
#include "areaseries.h"
#include "environment.h"
#include "chartaxispen.h"
#include "toollist.h"
#include "tools.h"
#include "markstiptool.h"
#include "ValueList.h"
#include <math.h>
#include <set>
#include "smithseries.h"
#include "polarseries.h"

//#include "valuelist.h"

//#include "TraceLib/STArlogger.h"	// logger include

const int CChartWrapper::LINE_TYPE = 0;
const int CChartWrapper::BAR_TYPE = 1;
const int CChartWrapper::H_BAR_TYPE = 2;
const int CChartWrapper::POINT_TYPE = 4;
const int CChartWrapper::BEZIER_TYPE = 16;
const int CChartWrapper::AREA_TYPE = 3;
const int CChartWrapper::SMITH_TYPE = 34;
const int CChartWrapper::POLAR_TYPE = 12;

CChartWrapper::CChartWrapper(CTChart *chart) : pchart_(chart),
plot_count_(0),
isAreaActive_(false),
bAutoscale_(false),
isDynamicallyCreated_(false)
{
}

CChartWrapper::~CChartWrapper(void)
{
	if ( font_.get() ) {
		font_->DeleteObject();
	}

	if (isDynamicallyCreated_)
	{
		if (pchart_) {
			delete pchart_;
			pchart_ = 0;
		}
	}
	
}

CTChart* CChartWrapper::GetTChart()
{
	return pchart_;
}

bool CChartWrapper::CreateTChart(const CRect &rect,
								CWnd *parent,
								const UINT res_id)
{
	pchart_ = new CTChart();
	try
	{
		BOOL bStatus{ FALSE };

		while (true) {

			bStatus = pchart_->Create(_T(""), _T(""), WS_CHILD | WS_VISIBLE, rect, parent, res_id);

			if (bStatus) {
				break;
			}

			::Sleep(50);
		}

		if (bStatus) {

			while (true) {

				if (pchart_->GetSafeHwnd()) {
					break;
				}

				::Sleep(100);
			}
		}

		res_id_ = res_id;
	}
	catch(...)
	{
	}

	//pchart_->RemoveAllSeries();

	isDynamicallyCreated_ = true;
	
	return true;
}


int CChartWrapper::AddPlot(const CString &strParameter,
						   const double width, 
						   const long axis)
{
	int plot_id = pchart_->AddSeries(LINE_TYPE);

	return plot_id;
}

int CChartWrapper::AddLinePointPlotEx(const CString &strParameter,
						        const double width,
								const long axis)
{
	int plot_id = AddPlot(strParameter, width, axis);
	return plot_id;
}


void CChartWrapper::RemoveAllPlots()
{
	long sz = pchart_->GetSeriesCount();

	if (sz > 0 ) {
		pchart_->RemoveAllSeries();
	}
}

void CChartWrapper::DestroyChartWindow()
{
	if (isDynamicallyCreated_ && pchart_->GetSafeHwnd() != NULL) {
		pchart_->SendMessage(WM_CLOSE);
		pchart_->DestroyWindow();
	}
}

HWND CChartWrapper::GetSafeHwnd()
{
	return pchart_->GetSafeHwnd();
}
