#pragma once
#include <set>
#include <map>
#include <vector>
#include <memory>
#include "helpers\TeeChartDefines.h"
#include "datatypes.h"

class CTChart;

class CSmithSeries;
class CPolarSeries;

namespace nsSTAr
{
	namespace nsChartDataType
	{
		typedef std::vector<double> CReal64Vector;

		struct CColor
		{
			COLORREF color_;

			CColor(const long r,const long g, const long b) : color_(RGB(r,g,b)) {}

			CColor(const COLORREF color) : color_(color) {}

			CColor& operator=(const CColor &source)
			{
				color_ = source.color_;
			}

			COLORREF getColor() 
			{
				return color_;
			}
		};

		enum enChartLineStyle
		{
			LineNone                  = psClear,
            // Solid line.
            LineSolid                 = psSolid,
            // Dash line.
            LineDash                  = psDash,
            // Dotted line.
            LineDot                   = psDot,
            // Dash-dot line.
            LineDashDot               = psDashDot,
            // Dash-dot-dot line.
            LineDashDotDot            = psDashDotDot,
            // Step centered line.
            LineSmallDot			  =	psSmallDots,

			LineStepXY				  = 8,
			
			LineStepYX				  = 9
			
		};

		enum enChartPointStyle
		{
			PointNone                 = 0,
            // Empty square.
            PointEmptySquare          = 1,
            // Solid square.
            PointSolidSquare          = 2,
            // Asterisk
            PointAsterisk             = 3,
            // Dotted empty square.
            PointDottedEmptySquare    = 4,
            // Dotted solid square.
            PointDottedSolidSquare    = 5,
            // Solid diamond.
            PointSolidDiamond         = 6,
            // Empty square with X.
            PointEmptySquareWithX     = 7,
            // Empty square with cross.
            PointEmptySquareWithCross = 8,
            // Empty circle.
            PointEmptyCircle          = 9,
            // Solid circle.
            PointSolidCircle          = 10,
            // Dotted empty circle.
            PointDottedEmptyCircle    = 11,
            // Dotted solid circle.
            PointDottedSolidCircle    = 12,
            // X
            PointX                    = 13,
            // Bold X.
            PointBoldX                = 14,
            // Small X.
            PointSmallX               = 15,
            // Cross
            PointCross                = 16,
            // Bold cross.
            PointBoldCross            = 17,
            // Small cross.
            PointSmallCross           = 18,
            // Small empty square.
            PointSmallEmptySquare     = 19,
            // Small solid square.
            PointSmallSolidSquare     = 20,
            // Simple dot.
            PointSimpleDot            = 21,
            // Empty diamond.
            PointEmptyDiamond         = 22 			
		};	

		enum enChartPointerStyle
		{
			PointerNone             = -1,
			PointerSquare			= psRectangle,
			PointerCircle				= psCircle,
			PointerCross				= psCross,
            PointerX					= psDiagCross,
            PointerAsterisk         = psStar,
            PointerDiamond			= psDiamond,   
            PointerSimpleDot          = psSmallDot
		};
	}
}

class CChartWrapper
{
public:
	
	CChartWrapper(CTChart *chart = 0);
	~CChartWrapper(void);

	static const int LINE_TYPE;
	static const int BAR_TYPE;
	static const int H_BAR_TYPE;
	static const int POINT_TYPE;
	static const int BEZIER_TYPE;
	static const int AREA_TYPE;
	static const int SMITH_TYPE;
	static const int POLAR_TYPE;

	std::auto_ptr<CFont> font_;
	std::map<int,COLORREF> map_plot_line_color_;
	std::map<int,COLORREF> map_plot_point_color_;
	std::map<int,COLORREF> map_plot_area_color_;

	long nPointStyle_;

	enum enAxis
	{
		EN_AXIS_NONE		 = -1,
		EN_AXIS_VERTICAL_LEFT = 0,
		EN_AXIS_HORIZONTAL_TOP = 1,
		EN_AXIS_VERTICAL_RIGHT = 2,
		EN_AXIS_HORIZONTAL_BOTTOM = 3		
	};

	enum enVerticalAxis
	{
		EN_VERTICAL_AXIS_LEFT = 0,
		EN_VERTICAL_AXIS_RIGHT = 1,
		EN_VERTICAL_AXIS_BOTH = 2		
	};

	enum enImageFormat
	{
		EN_FORMAT_NONE = 0,
		EN_FORMAT_BMP,
		EN_FORMAT_JPG,
		EN_FORMAT_PNG
	};

	enum enFrameStyle
	{
		EN_FRAME_NONE = 0,
		EN_FRAME_CLASSIC
	};

	enum enLableStyle
	{
		EN_LABLE_NONE = 0,
		EN_LABLE_NORMAL,
		EN_LABLE_NEGETIVE_EXPONENTIAL,
		EN_LABLE_POSITIVE_EXPONENTIAL
	};

	std::set<int>	set_axes_;
	enLableStyle xExpo_;
	enLableStyle y1Expo_;
	enLableStyle y2Expo_;

	std::map<int, std::shared_ptr<CSmithSeries> > mapSmithChartWnd_;
	std::map<int, std::shared_ptr<CPolarSeries> > mapPolarChartWnd_;
	

	UINT res_id_;

	CTChart* GetTChart();

	bool CreateTChart(const CRect &rect,
					CWnd *parent,
					const UINT res_id);
	
	bool SetTChartProperty();

	void GetTChartWindowRect(CRect& rect);

	void TChartInvalidate();

	int AddPlot(const CString &strParameter,const double width,const long axis = 0);

	int AddLinePointPlotEx(const CString &strParameter,
						        const double width,
								const long axis = 0);
	
	void RemoveAllPlots();

	
	void DestroyChartWindow();
	
	HWND GetSafeHwnd();

private:
	CTChart *pchart_;
	bool isDynamicallyCreated_;
	int plot_count_;
	std::set<int>		set_plot_;
	std::set<int>		set_point_plot_;
	std::set<int>		set_line_point_plot_;
	std::map<CString, int>  mapOfLegends_;
	std::map<double,std::pair<long,long> >	mapIndex_;
	int FHozFactor_;
	int FVerFactor_;
	int lineStyle_;
	bool isAreaActive_;
	bool bAutoscale_;
	int prev_plotid_;
	bool bInitX_;
	bool bInitY1_;
	bool bInitY2_;
};

