
// SampleChart.h : main header file for the PROJECT_NAME application
//

#pragma once

#ifndef __AFXWIN_H__
	#error "include 'stdafx.h' before including this file for PCH"
#endif

#include "resource.h"		// main symbols
#include "datatypes.h"


// CSampleChartApp:
// See SampleChart.cpp for the implementation of this class
//

struct CStAxisProperty
{
	std::string m_strGraphName;
	std::string m_strXAxisName;
	std::string m_strY1AxisName;
	std::string m_strY2AxisName;
	std::string m_strComment1;
	double m_dXAxisMin;
	double m_dY1AxisMin;
	double m_dY2AxisMin;
	double m_dXAxisMax;
	double m_dY1AxisMax;
	double m_dY2AxisMax;
	bool m_bXScale;
	bool m_bY1Scale;
	bool m_bY2Scale;
	bool m_bAutoScale;
	int m_iUpdateMode;
};

struct CStLineProperty
{
	int m_nXY1LineColor;
	int m_nXY1LineStyle;
	int m_nXY1LineWidth;
	int m_nXY1PointStyle;
	int m_nXY1PointColor;
	int m_nXY1PointSize;

	int m_nXY2LineColor;
	int m_nXY2LineStyle;
	int m_nXY2LineWidth;
	int m_nXY2PointStyle;
	int m_nXY2PointColor;
	int m_nXY2PointSize;

	CStLineProperty();
	CStLineProperty(const CStLineProperty& source);
	CStLineProperty& operator = (const CStLineProperty& source);
	bool operator == (const CStLineProperty& source1) const;
	~CStLineProperty();
};

class CStGIS_AnalyzerData
{
public:

	std::string m_strTestName;
	std::string m_strCommandName;
	std::string m_strTesterID;

	// Window coordinates 
	int m_nSizeInPercentage;
	int m_nXCoord;
	int m_nYCoord;
	int m_nWidth;
	int m_nHeight;

	int enable_close_;

	// sub window counts
	int m_nTextCount;
	int m_nGraphcount;
	CStGIS_AnalyzerData();
	~CStGIS_AnalyzerData();
	CStGIS_AnalyzerData(const CStGIS_AnalyzerData& source);
	CStGIS_AnalyzerData& operator = (const CStGIS_AnalyzerData& source);

	////std::basic_string<unsigned char> Encode() const;
	////void Decode(std::basic_string<unsigned char> &strPacked);
};

class CStGIS_GraphData
{
public:

	std::string m_strTestName;
	std::string m_strCommandName;
	std::string m_strTesterID;

	int m_nSequenceID;		//Sequence ID  for  subwindow: 
	int m_nGraphDataType; /*
						  0 - Axis Property ->  X-Y
						  1 - Axis Property -> X-Y1-Y2
						  2 - Line Property -> X-Y
						  3 - Line Property -> X-Y1-Y2
						  4 - single point X-Y
						  5 - single point X-Y1-Y2
						  6 - 1D Array X-Y
						  7 - 1D Array X-Y1-Y2
						  8 - 2D Array X-Y
						  9 - 2D Array X-Y-Y2
						  */

	CStAxisProperty m_stObjProp;
	CStLineProperty m_stObjLine;

	double m_dXSingleXY1Value;
	double m_dXSingleXY2Value;
	double m_dXSingleXValue;

	// data storage 
	Array1D_Double m_vec_XAxisValues;
	Array1D_Double m_vec_Y1AxisValues;
	Array1D_Double m_vec_Y2AxisValues;

	Array2D_Double m_vec_2DY1AxisValues;
	Array2D_Double m_vec_2DY2AxisValues;

	int m_nLineColor;
	int m_nLine2Color;
	int m_nDataUpdateMode; // 0 - Append -, 1 Clear For Graph 
	int m_nXYColor;
	int m_nXY2Color;

	CStGIS_GraphData();
	virtual ~CStGIS_GraphData();
	CStGIS_GraphData(const CStGIS_GraphData& source);
	CStGIS_GraphData& operator = (const CStGIS_GraphData& source);

	////// overrides
	////std::basic_string<unsigned char> Encode() const;
	////void Decode(std::basic_string<unsigned char> &strPacked);

};

class CStGIS_TextData 
{
public:

	std::string m_strTestName;
	std::string m_strCommandName;
	std::string m_strTesterID;

	int m_nTextDataType;
	int m_nSequenceID;


	double m_dSingleValues;
	int m_nSingleValues;
	int m_nUpdateMode;

	Array1D_String m_vec_paramname;
	Array2D_Double m_vec_2DValues;

	CStGIS_TextData();
	virtual ~CStGIS_TextData();
	CStGIS_TextData(const CStGIS_TextData& source);
	CStGIS_TextData& operator = (const CStGIS_TextData& source);

	////// overrides
	////std::basic_string<unsigned char> Encode() const;
	////void Decode(std::basic_string<unsigned char> &strPacked);

};

struct CPlot_Data
{
	std::deque<double> deq_dXAxisVal;
	std::deque<double> deq_dYAxisVal;
	std::deque<double> deq_dY2AxisVal;
	std::deque<Array1D_Double> deq_d1XAxisVal;
	std::deque<Array1D_Double> deq_d1YAxisVal;
	std::deque<Array1D_Double> deq_d1Y2AxisVal;
	std::deque<Array2D_Double> deq_d2XAxisVal;
	std::deque<Array2D_Double> deq_d2YAxisVal;
	std::deque<Array2D_Double> deq_d2Y2AxisVal;

	std::deque<double>::iterator dblX_iter_;
	std::deque<double>::iterator dblY_iter_;
	std::deque<double>::iterator dblY2_iter_;

	std::deque<Array1D_Double>::iterator dblArX_iter_;
	std::deque<Array1D_Double>::iterator dblArY_iter_;
	std::deque<Array1D_Double>::iterator dblArY2_iter_;

	std::deque<Array2D_Double>::iterator dblAr2X_iter_;
	std::deque<Array2D_Double>::iterator dblAr2Y_iter_;
	std::deque<Array2D_Double>::iterator dblAr2Y2_iter_;

	int update;

	CStLineProperty line_prop;

	int plot_line_id_;

	bool operator==(const CPlot_Data& source1) const;

};

struct CPlotTextInfo
{
	enum E_GIS_Command_Set gis_type_;
	int sequenceID_;
	std::string paramName1_;
	int nSingleValue_;
	int nUpdateMode_;
	double dSingleValue_;
	Array1D_Double temp_;
	std::string paramName2_;
	Array1D_Double temp1_;
	std::string paramName3_;
	Array1D_Double temp2_;
	Array2D_Double temp3_;

	CPlotTextInfo()	{}

	~CPlotTextInfo()	{}
};

struct CPlotGraphInfo
{
	enum E_GIS_Command_Set gis_cmd_;
	int nSequenceID_;
	CStAxisProperty stObjProp_;
	CStLineProperty stObjLine_;
	int nGraphDataType_;

	double dXSingleXY1Value_;
	double dXSingleXY2Value_;
	double dXSingleXValue_;

	// data storage 
	Array1D_Double vec_XAxisValues_;
	Array1D_Double vec_Y1AxisValues_;
	Array1D_Double vec_Y2AxisValues_;

	Array2D_Double vec_2DY1AxisValues_;
	Array2D_Double vec_2DY2AxisValues_;

	int nLineColor_;
	int nLine2Color_;
	int nDataUpdateMode_; // 0 - Append -, 1 Clear For Graph 
	int nXYColor_;
	int nXY2Color_;

	CPlotGraphInfo()	{}

	~CPlotGraphInfo()	{}
};


class CSampleChartApp : public CWinApp
{
public:
	CSampleChartApp();

	CCriticalSection csgraph_;

	void CreateGraphThreads();

private:
	

// Overrides
public:
	virtual BOOL InitInstance();

	

// Implementation

	DECLARE_MESSAGE_MAP()


};

extern CSampleChartApp theApp;