// StAnalyzerDlg.cpp : implementation file
//
#include "StdAfx.h"
#include "SampleChart.h"
////#include "StAnalyzerManager.h"
#include "StAnalyzerDlg.h"
#include <deque>
#include <map>
#include <algorithm>
#include <limits>
#include <float.h>
#include "StGisCommands.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#define MIN_WINDOWX 350
#define MIN_WINDOWY 350

#define PLOT_6	6
#define PLOT_12	12
#define IDC_TCHART1  501

//////////////////////////////////////////////////////////////////////////////////////////////
////
int CStAnalyzerDlg::object_count_ = 0;
const int CStAnalyzerDlg::pad_ = 10;
const int CStAnalyzerDlg::graph_height_ = 150;
const int CStAnalyzerDlg::graph_width_ = 150;
const int CStAnalyzerDlg::max_no_graph_per_row_ = 4;


IMPLEMENT_DYNAMIC(CStAnalyzerDlg, CDialog)
CStAnalyzerDlg::CStAnalyzerDlg(LPCTSTR tzTest, LPCTSTR tzTester, CWnd* pParent /*=NULL*/)
: CDialog(CStAnalyzerDlg::IDD, pParent),
val1_(true),
val2_(true)
{

	this->m_strTest = tzTest;
	m_strTester = tzTester;
	m_nTextResourceID = 200;
	m_nGraphResourceID = IDC_TCHART1;
    ++object_count_;
    text_control_count_ = 0;
    graph_control_count_ = 0;
	last_plot_id_ = -1;
	prev_xy_color_ = -1;
	prev_xy2_color_ = -1;
}

//////////////////////////////////////////////////////////////////////////////////////////////
////

CStAnalyzerDlg::~CStAnalyzerDlg()
{
	TRACE("\nDestroying CStAnalyzerDlg\n");
}

//////////////////////////////////////////////////////////////////////////////////////////////
////

void CStAnalyzerDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

//////////////////////////////////////////////////////////////////////////////////////////////
////

BEGIN_MESSAGE_MAP(CStAnalyzerDlg, CDialog)

	ON_WM_CLOSE()
	ON_WM_CTLCOLOR()
	
	ON_WM_DESTROY()
	ON_WM_GETMINMAXINFO()

	ON_MESSAGE(UWM_RESET_DLG_CONTENT,OnResetDlgContent)
	ON_MESSAGE(UWM_CREATE_SUB_WINDOW, OnCreateSubWindow)
END_MESSAGE_MAP()

//////////////////////////////////////////////////////////////////////////////////////////////
////

LRESULT CStAnalyzerDlg::OnResetDlgContent(WPARAM wParam, LPARAM lParam)
{
	std::unique_ptr<std::pair<int, int>> pr{ reinterpret_cast<std::pair<int, int> *>(lParam) };

	ResetDialogContent();

	return 0L;
}

BOOL CStAnalyzerDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	CString strTime, strText;
	GetTime(strTime);

	count_plot_line_ = 0;
	is_force_close_ = false;

	return FALSE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CStAnalyzerDlg::OnClose()
{
	if (is_force_close_ == true)
	{
		--object_count_;
		DestroyWindow();
		return;
	}

	std::auto_ptr<std::pair<std::string, std::string> > pPair(new std::pair<std::string, std::string>);
	pPair->first = std::string(m_strTest);
	pPair->second = std::string(m_strTester);
   	--object_count_;
	DestroyWindow();
}


///////////////////////////////////////////////////////////////////////////////////////////////////////
//

void CStAnalyzerDlg::GetTime(CString& strTime)
{
	CTime theTime;
	theTime = CTime::GetCurrentTime();
	strTime = theTime.Format("%Y%b%d %H:%M:%S");

}

nsSTAr::nsChartDataType::enChartLineStyle CStAnalyzerDlg::GetLineStyle(const int nLineStyle)
{
	return nsSTAr::nsChartDataType::LineSolid;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////
//

nsSTAr::nsChartDataType::enChartPointStyle CStAnalyzerDlg::GetPointStyle(const int nPointStyle)
{
	return nsSTAr::nsChartDataType::PointBoldX;
}

COLORREF CStAnalyzerDlg::GetColor(int nColor)
{
    int nColorIndex = ((nColor >= 0) && (nColor <= 12)) ? nColor : 13;

    static COLORREF arr_color[14] = {
        RGB(255,0,0),
        RGB(0,255,0),
        RGB(0,0,255),
        RGB(0,0,0),
        RGB(255,127,0),
        RGB(255,255,125),
        RGB(0,0,160),
        RGB(128,0,255),
        RGB(192,192,192),
        RGB(64,0,128),
        RGB(255,0,255),
		RGB(255,128,192),
        RGB(0,255,255),
        // default
        RGB(255,0, 255)

    };

    return arr_color[nColorIndex];	
}

void CStAnalyzerDlg::PostNcDestroy()
{
	CDialog::PostNcDestroy();
	
}

void CStAnalyzerDlg::OnGetMinMaxInfo(MINMAXINFO* lpMMI)
{
	// TODO: Add your message handler code here and/or call default
	lpMMI->ptMinTrackSize.x = MIN_WINDOWX;
	lpMMI->ptMinTrackSize.y = MIN_WINDOWY;
	CDialog::OnGetMinMaxInfo(lpMMI);
}


void CStAnalyzerDlg::OnDestroy()
{
	for(std::map<int, CEdit*>::iterator itr = m_aMapTxtWndToSequence.begin();
									   itr != m_aMapTxtWndToSequence.end();
									   ++itr)
	{
		itr->second->DestroyWindow();
		delete itr->second;
	}
	m_aMapTxtWndToSequence.clear();

	for(std::vector<HFONT>::iterator itr = vec_font_.begin();
									itr != vec_font_.end();
									++itr)
	{
		DeleteObject(*itr);
	}
	vec_font_.clear();

	for(std::map<int, CChartWrapper*>::iterator itr = m_aMapGrpWndToSequence.begin();
												itr != m_aMapGrpWndToSequence.end();
												++itr)
	{
		itr->second->RemoveAllPlots();
		itr->second->DestroyChartWindow();
		delete itr->second;
		m_nGraphResourceID--;
	}
	m_aMapGrpWndToSequence.clear();

	CDialog::OnDestroy();
}

void CStAnalyzerDlg::ResetDialogContent()
{
	COLORREF color = RGB(255, 255, 255);
	COLORREF frameColor = RGB(225, 225, 225);

	for(std::map<int, CChartWrapper*>::iterator itr = m_aMapGrpWndToSequence.begin();
												itr != m_aMapGrpWndToSequence.end();
												++itr)
	{

		itr->second->RemoveAllPlots();

		itr->second->AddLinePointPlotEx(_T(""), 1);
	}
}

LRESULT CStAnalyzerDlg::OnCreateSubWindow(WPARAM wparam, LPARAM lparam)
{
	bool *status = reinterpret_cast<bool *>(wparam);
	std::pair<int, int> *pr(reinterpret_cast<std::pair<int, int> *>(lparam));
	*status = CreateSubWindow(pr->first, pr->second);

	return 0L;
}

bool CStAnalyzerDlg::CreateSubWindow(int nTextWindow, int nGraphWindow)
{
	int m_nWndCount = 0;
	m_nWndCount = nTextWindow + nGraphWindow;

	if(m_nWndCount > 16)
	{
		return false;
	}
	
    std::vector<CRect> pRect;
    CRect clientrect;

    if(GetSafeHwnd() != NULL)
	{
		GetClientRect(&clientrect);
		clientrect.NormalizeRect();
		clientrect.top = pad_;
		clientrect.left = pad_;
		clientrect.bottom = clientrect.bottom - pad_;
		clientrect.right = clientrect.right - pad_;
        GetDimensions(pRect,clientrect);
    }

	COLORREF color = RGB(255,255,255);
	COLORREF frameColor = RGB(225, 225, 225);

	int nGraphCount = nTextWindow + nGraphWindow;
	for (int n = 0; nTextWindow < nGraphCount; nTextWindow++, n++, m_nGraphResourceID++)
	{			
		CChartWrapper* pCtrlGraph = new CChartWrapper();
		pCtrlGraph->CreateTChart(pRect[nTextWindow], this, m_nGraphResourceID);
		m_aMapGrpWndToSequence.insert(std::make_pair(n, pCtrlGraph));

	}

	return true;
}

void CStAnalyzerDlg::GetDimensions(std::vector<CRect>& vecdimensions, const CRect& windowdimension)
{
	CRect rectdimension;

    int graph_width, graph_height;

    int control_per_row = 0;

    int control_count = text_control_count_ + graph_control_count_;

    GetControlDimension(windowdimension, graph_width, graph_height);
 
	for(int n = 0; n < control_count; n++)
	{
        switch(n)
        {
        case 0:
            rectdimension.left = windowdimension.left;
			rectdimension.top  = windowdimension.top;
			rectdimension.right = graph_width + pad_;
			rectdimension.bottom = graph_height + pad_;
			vecdimensions.push_back(rectdimension);
            break;

        case 1:
            rectdimension.left = (vecdimensions.at(n-1)).right;
            rectdimension.top  =  (vecdimensions.at(n-1)).top;
            rectdimension.right = rectdimension.left + graph_width;
            rectdimension.bottom = graph_height + pad_;
            vecdimensions.push_back(rectdimension);
            break;
        case 2:
			{
                switch(control_count)
                {
                    case 3:
                    case 4:
                        control_per_row = 2;
                        rectdimension.left = (vecdimensions.at(0)).left;
                        rectdimension.top  =  (vecdimensions.at(0)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                    break;
                    case 5:
                    case 6:
                    case 7:
                    case 8:
                    case 9:
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                    case 16:
                        control_per_row = 3;
                        rectdimension.left = (vecdimensions.at(n - 1)).right;
                        rectdimension.top  =  (vecdimensions.at(n - 1)).top;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                    break;
                }
            }
            break;
        case 3:
            {
                switch(control_count)
                {
                    
                    case 4:
                    
                        control_per_row = 2;
                        rectdimension.left = (vecdimensions.at(1)).left;
                        rectdimension.top  =  (vecdimensions.at(1)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                    break;
                    case 5:
                    case 6:
                    case 7:
                    case 8:
                    case 9:
                        rectdimension.left = (vecdimensions.at(0)).left;
                        rectdimension.top  =  (vecdimensions.at(0)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                    break;
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                    case 16:
                        control_per_row = 3;
                        rectdimension.left = (vecdimensions.at(n - 1)).right;
                        rectdimension.top  =  (vecdimensions.at(n - 1)).top;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                    break;
                }
            }
            break;
        case 4:
            {
                switch(control_count)
                {
                    
                    case 5:
                    case 6:
                    case 7:
                    case 8:
                    case 9:
                        control_per_row = 2;
                        rectdimension.left = (vecdimensions.at(1)).left;
                        rectdimension.top  =  (vecdimensions.at(1)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                    break;
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                    case 16:
                        rectdimension.left = (vecdimensions.at(0)).left;
                        rectdimension.top  =  (vecdimensions.at(0)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                    break;
                }
            }
            break;
        case 5:
            {
                switch(control_count)
                {
                    case 6:
                    case 7:
                    case 8:
                    case 9:    
                        rectdimension.left = (vecdimensions.at(2)).left;
                        rectdimension.top  =  (vecdimensions.at(2)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                    break;
                    
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                    case 16:
                        rectdimension.left = (vecdimensions.at(1)).left;
                        rectdimension.top  =  (vecdimensions.at(1)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                        break;
                }
            }
            break;
        
        case 6:
            {
                switch(control_count)
                {
                    
                    case 7:
                    case 8:
                    case 9:    
                        rectdimension.left = (vecdimensions.at(3)).left;
                        rectdimension.top  =  (vecdimensions.at(3)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                    break;
                    
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                    case 16:
                        rectdimension.left = (vecdimensions.at(2)).left;
                        rectdimension.top  =  (vecdimensions.at(2)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                        break;
                }
            }
            break;
        case 7:
            {
                switch(control_count)
                {
                    
                    case 8:
                    case 9:    
                        rectdimension.left = (vecdimensions.at(4)).left;
                        rectdimension.top  =  (vecdimensions.at(4)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                    break;
                    
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                    case 16:
                        rectdimension.left = (vecdimensions.at(3)).left;
                        rectdimension.top  =  (vecdimensions.at(3)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                        break;
                }
            }
            break;
        case 8:
            {
                switch(control_count)
                {
                    
                    case 9:    
                        rectdimension.left = (vecdimensions.at(5)).left;
                        rectdimension.top  =  (vecdimensions.at(5)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                    break;
                    
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                    case 16:
                        rectdimension.left = (vecdimensions.at(4)).left;
                        rectdimension.top  =  (vecdimensions.at(4)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                        break;
                }
            }
            break;
        case 9:
            {
                switch(control_count)
                {
                    
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                    case 16:
                        rectdimension.left = (vecdimensions.at(5)).left;
                        rectdimension.top  =  (vecdimensions.at(5)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                        break;
                }
            }
            break;
        case 10:
            {
                switch(control_count)
                {
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                    case 16:
                        rectdimension.left = (vecdimensions.at(6)).left;
                        rectdimension.top  =  (vecdimensions.at(6)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                        break;
                }
            }
            break;
        case 11:
            {
                switch(control_count)
                {
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                    case 16:
                        rectdimension.left = (vecdimensions.at(7)).left;
                        rectdimension.top  =  (vecdimensions.at(7)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                        break;
                }
            }
            break;
            
        case 12:
            {
                switch(control_count)
                {
                    
                    case 13:
                    case 14:
                    case 15:
                    case 16:
                        rectdimension.left = (vecdimensions.at(8)).left;
                        rectdimension.top  =  (vecdimensions.at(8)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                        break;
                }
            }
            break;
        case 13:
            {
                switch(control_count)
                {
                    
                    case 14:
                    case 15:
                    case 16:
                        rectdimension.left = (vecdimensions.at(9)).left;
                        rectdimension.top  =  (vecdimensions.at(9)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                        break;
                }
            }
            break;
        case 14:
            {
                switch(control_count)
                {
                   
                    case 15:
                    case 16:
                        rectdimension.left = (vecdimensions.at(10)).left;
                        rectdimension.top  =  (vecdimensions.at(10)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                        break;
                }
            }
            break;
        case 15:
            {
                switch(control_count)
                {
                    case 16:
                        rectdimension.left = (vecdimensions.at(11)).left;
                        rectdimension.top  =  (vecdimensions.at(11)).bottom;
                        rectdimension.right = rectdimension.left + graph_width;
                        rectdimension.bottom = rectdimension.top + graph_height;
                        vecdimensions.push_back(rectdimension);
                        break;
                }
            }
            break;
                    
        }
    }
}

void CStAnalyzerDlg::GetControlWidth(const CRect &windowdimension, int &graph_width,const int total_wnd_count)
{
    switch(total_wnd_count)
    {
        case 1:
            graph_width = windowdimension.Width();
            break;
        case 2:
            graph_width = windowdimension.Width() / 2;
            break;
        case 3:
        case 4:
            graph_width = windowdimension.Width() / 2;
            break;
        case 5:
        case 6:
            graph_width = windowdimension.Width() / 3;
            break;
        case 7:
        case 8:
        case 9:
            graph_width = windowdimension.Width() / 3;
            break;
        case 10:
        case 11:
        case 12:
            graph_width = windowdimension.Width() / 4;
            break;
        case 13:
        case 14:
        case 15:
        case 16:
            graph_width = windowdimension.Width() / 4;
            break;
        default:;
    }
}

void CStAnalyzerDlg::GetControlHeight(const CRect &windowdimension, int &height,const int total_wnd_count)
{
    switch(total_wnd_count)
    {
        case 1:
            height = windowdimension.Height();
            break;
        case 2:
            height = windowdimension.Height();
            break;
        case 3:
        case 4:
            height = windowdimension.Height() / 2;
            break;
        case 5:
        case 6:
            height = windowdimension.Height() / 2;
            break;
        case 7:
        case 8:
        case 9:
            height = windowdimension.Height() / 3;
            break;
        case 10:
        case 11:
        case 12:
            height = windowdimension.Height() / 3;
            break;
        case 13:
        case 14:
        case 15:
        case 16:
            height = windowdimension.Height() / 4;
            break;
        default:;
    }
}

void CStAnalyzerDlg::GetControlDimension(const CRect &windowdimension, int &graph_width, int &graph_height)
{
    int control_count = text_control_count_ + graph_control_count_;

    GetControlWidth(windowdimension,graph_width,control_count);
    GetControlHeight(windowdimension,graph_height,control_count);

	//LOG_DEBUG << "GetControlDimension" << ENDLOG;
}

BEGIN_EVENTSINK_MAP(CStAnalyzerDlg, CDialog)
END_EVENTSINK_MAP()
