#include "StdAfx.h"
#include "SampleChart.h"
#include "StGISCmdStr.h"
#include "StGISCommands.h"
#include "StAnalyzerDlg.h"
#include "StAnalyzerManager.h"
#include "atlcoll.h"
#include "datatypes.h"
#include "StAnalyzerThread.h"
#include <functional>
#include <algorithm>

CStAnalyzerManager* CStAnalyzerManager::pStAnalyzerWindowDispatcher = 0;

///////////////////////////////////////////////////////////////////////////////////

const std::string CTesterAnalyzerManager::TESTER1_("Tester#1");

CTesterAnalyzerManager::CTesterAnalyzerManager(void)
{
	map_tester_results_.insert(std::make_pair(TESTER1_, std::shared_ptr<CTesterAnalyzer>(new CTesterAnalyzer(TESTER1_))));
}

CTesterAnalyzerManager::~CTesterAnalyzerManager(void)
{
}

/////////////////

CTesterAnalyzerManager::CTesterAnalyzer::CTesterAnalyzer(const std::string &testerid)
{
}

CTesterAnalyzerManager::CTesterAnalyzer& CTesterAnalyzerManager::GetTesterAnalyzer(const std::string& testerid)
{
	IterMap iter = map_tester_results_.find(testerid);

	if(iter == map_tester_results_.end())
	{
		map_tester_results_.insert(std::make_pair(testerid, std::shared_ptr<CTesterAnalyzer>(new CTesterAnalyzer(testerid))));

		iter = map_tester_results_.find(testerid);
	}

	if(iter == map_tester_results_.end())
	{
		throw std::runtime_error("Unexpected mismatch in map");
	}

	return *(iter->second.get());
}

////////////////////////////////////////////////////////////////

CStAnalyzerManager::CStAnalyzerManager(void)
{	
	m_nPrefCount = 16;
    previous_test_name_ = "";
	previous_tester_id_ = "";

	for(int nValue = 0 ; nValue < static_cast<int>(GetGISTotalCommands()); nValue++)
	{
		arrMapGisCommands.SetAt(GIS_CommandString[nValue],nValue);
	}

	nResourceID_ = 301;
}

///////////////////////////////////////////////////////////////////////
//

CStAnalyzerManager::~CStAnalyzerManager(void)
{
	std::map<std::pair<std::string, std::string>, CStAnalyzerDlg*>::const_iterator iter;
	//std::map<std::pair<std::string, std::string>, CStAnalyzerThread*>::const_iterator iter;

    for(iter = m_aMapGISWndToAlgo.begin(); iter != m_aMapGISWndToAlgo.end(); ++iter)
    {
        HWND hwnd = iter->second->GetSafeHwnd();
		//HWND hwnd = iter->second->m_pMainWnd->GetSafeHwnd();
		if (IsWindow(hwnd))
			iter->second->DestroyWindow();
			//iter->second->CloseWindow();
        delete iter->second;
    }
	arrMapGisCommands.RemoveAll();

}

//////////////////////////////////////////////////////////////////////
//

void CStAnalyzerManager::CreateAnalyzerWindow (LPCTSTR tzTest, LPCTSTR tzTester, CRect rect, int nTextCount, int nGraphcount)
{
	CSingleLock cslock(&theApp.csgraph_);
	cslock.Lock();

	std::map<std::pair<std::string, std::string>, CStAnalyzerDlg*>::iterator itr = m_aMapGISWndToAlgo.find(std::make_pair(tzTest, tzTester));
	//std::map<std::pair<std::string, std::string>, CStAnalyzerThread*>::iterator itr = m_aMapGISWndToAlgo.find(std::make_pair(tzTest, tzTester));
	if(itr != m_aMapGISWndToAlgo.end())
	{
		itr->second->SendMessage(UWM_RESET_DLG_CONTENT);
		//itr->second->m_pMainWnd->SendMessage(UWM_RESET_DLG_CONTENT);
		return;
	}

	CStAnalyzerDlg* pGraphWnd = new CStAnalyzerDlg (tzTest, tzTester, AfxGetApp()->m_pMainWnd);
	//CStAnalyzerThread* pThread = dynamic_cast<CStAnalyzerThread *>(AfxBeginThread(RUNTIME_CLASS(CStAnalyzerThread),
	//																THREAD_PRIORITY_NORMAL,
	//																0,
	//																CREATE_SUSPENDED));

	//CEvent event;

	//pThread->SetCreateEvent(&event);
	//pThread->SetParent(AfxGetApp()->m_pMainWnd);
	//pThread->SetAnalyzerDlgInfo(tzTest, tzTester, rect, nTextCount, nGraphcount);

	//pThread->m_bAutoDelete = FALSE;
	//pThread->ResumeThread();

	//event.Lock();

	if (pGraphWnd->GetSafeHwnd() == 0)
	{
		if ( !pGraphWnd->Create(CStAnalyzerDlg::IDD,NULL))
		{
            TRACE("Window creation failed\n");
		}
        else
        {
            pGraphWnd->text_control_count_ = nTextCount;
            pGraphWnd->graph_control_count_ = nGraphcount;
        }
	}
	else
	{
		pGraphWnd->text_control_count_ = nTextCount;
        pGraphWnd->graph_control_count_ = nGraphcount;
	}

	pGraphWnd->ShowWindow(SW_HIDE);
	pGraphWnd->UpdateWindow();
	pGraphWnd->MoveWindow(rect);

	//if(pGraphWnd->CreateSubWindow(nTextCount, nGraphcount))

	bool status = false;
	std::pair<int, int> pr;
	pr.first = nTextCount;
	pr.second = nGraphcount;
	pGraphWnd->SendMessage(UWM_CREATE_SUB_WINDOW, (WPARAM)&status, (LPARAM)&pr);

	if (status)
	{
		pGraphWnd->ShowWindow(SW_SHOW);
		pGraphWnd->SetFocus();
		pGraphWnd->SetForegroundWindow();
	}
	
    std::string algoname = tzTest;
	std::string testerid = tzTester;

    //m_aMapGISWndToAlgo.insert(std::make_pair(algoname,pGraphWnd));
	m_aMapGISWndToAlgo.insert(std::make_pair(std::make_pair(algoname, testerid), pGraphWnd));
	//m_aMapGISWndToAlgo.insert(std::make_pair(std::make_pair(algoname, testerid), pThread));
}

void CStAnalyzerManager::HandleAnalyzerWndSpecificFunc(std::auto_ptr<CStGIS_AnalyzerData>& pData)
{
	CSingleLock cslock(&theApp.csgraph_);
	cslock.Lock();

	if (!pData.get()) {
		return;
	}

	int nCommandID{ -1 };
	if (arrMapGisCommands.Lookup(pData->m_strCommandName.c_str(), nCommandID) == FALSE) {
		return;
	}
	
	////CString value = 0;

	////if (STAr::CSTArPathManager::GetStringOption(CRegistrySettings::ANALYZER_WND_COUNT.c_str(), value, SAGI_REGISTRY_ROOT) == TRUE)
	////	m_nPrefCount = atoi(value);

	switch (nCommandID)
	{
		case GIS_CREATEANALYZERWINDOW:
		{
			if ((pData->m_nTextCount + pData->m_nGraphcount) > 16) {
				////CString str_message;
				////str_message.LoadString(IDS_REALTMANA_ERR_MAX_WINDOW);
				////theApp.errorhandler_.MessageBox(theApp.modulename_, str_message, SMB_ERROR);

			}
			else {
				CStAnalyzerThread* pAnalyzerThread{ GetAnalyzerThread(pData->m_strTestName, pData->m_strTesterID) };

				if (pAnalyzerThread == nullptr) {

					CRect rectSize;
					rectSize.left = pData->m_nXCoord;
					rectSize.top = pData->m_nYCoord;
					rectSize.right = pData->m_nWidth + rectSize.left;
					rectSize.bottom = pData->m_nHeight + rectSize.top;

					if (mapAnlyzerThreads_.size() >= m_nPrefCount) {
						RemoveAnalyzerThread(previous_test_name_, previous_tester_id_, true);
					}
					CreateAnalyzerThread(pData->m_strTestName.c_str(),
						pData->m_strTesterID.c_str(),
						rectSize,
						pData->m_nTextCount,
						pData->m_nGraphcount);

					previous_test_name_ = pData->m_strTestName;
					previous_tester_id_ = pData->m_strTesterID;

				}
				else {
					////pAnalyzerThread->PostThreadMessage(WM_ANALYZER_THREAD_RESET_DLG_CONTENT, 0, 0);
					pAnalyzerThread->ResetDialogContent();
				}
			}
		}
			break;


		case GIS_CREATEANALYZERWINDOW1:
		{
			if ((pData->m_nTextCount + pData->m_nGraphcount) > 16) {
				////CString str_message;
				////str_message.LoadString(IDS_REALTMANA_ERR_MAX_WINDOW);
				////theApp.errorhandler_.MessageBox(theApp.modulename_, str_message, SMB_ERROR);

			}
			else {
				CStAnalyzerThread* pAnalyzerThread{ GetAnalyzerThread(pData->m_strTestName, pData->m_strTesterID) };

				if (pAnalyzerThread == nullptr) {

					if (mapAnlyzerThreads_.size() >= m_nPrefCount) {
						RemoveAnalyzerThread(previous_test_name_, previous_tester_id_, true);
					}
					CreateAnalyzerThread(pData->m_strTestName.c_str(),
						pData->m_strTesterID.c_str(),
						pData->m_nSizeInPercentage,
						pData->m_nTextCount,
						pData->m_nGraphcount,
						true);

					previous_test_name_ = pData->m_strTestName;
					previous_tester_id_ = pData->m_strTesterID;

				}
				else {
					////pAnalyzerThread->PostThreadMessage(WM_ANALYZER_THREAD_RESET_DLG_CONTENT, 0, 0);
					pAnalyzerThread->ResetDialogContent();
				}
			}
		}
			break;

		case GIS_CLOSEANALYZERWINDOW:
		{
			RemoveAnalyzerThread(pData->m_strTestName, pData->m_strTesterID,true);
		}
			break;

		case GIS_SETWINDOW:
		{
			SetAnalyzerThreadDialogMenu(pData->m_strTestName, pData->m_strTesterID, pData->enable_close_);	
		}
			break;

		case GIS_CLEARALLDATA:
			break;

		default:
			break;	
	}
	
}

void CStAnalyzerManager::CreateAnalyzerThread(const std::string& strTestName, const std::string& strTesterID, int nSizeInPerc, int nTextCount, int nGraphcount, bool bCreateForeGround)
{
	CSingleLock csLock(&csMapAnalyzerThreads_);
	csLock.Lock();

	std::shared_ptr<CStAnalyzerThread> pAnalyzerThread{ CStAnalyzerThread::CreateAnalyzerThread(strTestName, strTesterID, nSizeInPerc, nTextCount, nGraphcount, nResourceID_,bCreateForeGround) };
	if (!pAnalyzerThread.get()) {
		throw;
	}

	mapAnlyzerThreads_.insert(std::make_pair(std::make_pair(strTestName, strTesterID), pAnalyzerThread));

	nResourceID_ += 100;

	csLock.Unlock();

}

void CStAnalyzerManager::CreateAnalyzerThread(const std::string& strTestName, const std::string& strTesterID, CRect rect, int nTextCount, int nGraphcount)
{
	CSingleLock csLock(&csMapAnalyzerThreads_);
	csLock.Lock();

	std::shared_ptr<CStAnalyzerThread> pAnalyzerThread{ CStAnalyzerThread::CreateAnalyzerThread(strTestName, strTesterID, rect, nTextCount, nGraphcount, nResourceID_) };
	if (!pAnalyzerThread.get()) {
		throw;
	}

	mapAnlyzerThreads_.insert(std::make_pair(std::make_pair(strTestName, strTesterID), pAnalyzerThread));

	nResourceID_ += 100;


	csLock.Unlock();
}

CStAnalyzerThread* CStAnalyzerManager::GetAnalyzerThread(const std::string& strTestName, const std::string& strTesterID)
{
	CStAnalyzerThread* pAnalyzerThread{ nullptr };
	CSingleLock csLock{ &csMapAnalyzerThreads_ };
	csLock.Lock();
	auto itr = mapAnlyzerThreads_.find(std::make_pair(strTestName, strTesterID));

	if (itr != mapAnlyzerThreads_.end()) {

		pAnalyzerThread = itr->second.get();

	}
	csLock.Unlock();

	return pAnalyzerThread;

}


void CStAnalyzerManager::RemoveAnalyzerThread(const std::string& strTestName, const std::string& strTesterID, bool bForceClose)
{
	CSingleLock csLock{ &csMapAnalyzerThreads_ };
	csLock.Lock();
	auto itr = mapAnlyzerThreads_.find(std::make_pair(strTestName, strTesterID));

	if (itr != mapAnlyzerThreads_.end()) {
		itr->second->PostThreadMessage(WM_ANALYZER_THREAD_SET_FORCE_CLOSE, 0, static_cast<LPARAM>(bForceClose));
		itr->second->PostThreadMessage(WM_QUIT, 0, 0);
		WaitForSingleObject(itr->second->m_hThread, INFINITE);
		mapAnlyzerThreads_.erase(itr);
	}
	csLock.Unlock();
}


void CStAnalyzerManager::SetAnalyzerThreadDialogMenu(const std::string& strTestName, const std::string& strTesterID, const int nButtonClose)
{
	CSingleLock csLock{ &csMapAnalyzerThreads_ };
	csLock.Lock();
	auto itr = mapAnlyzerThreads_.find(std::make_pair(strTestName, strTesterID));

	if (itr != mapAnlyzerThreads_.end()) {
		itr->second->PostThreadMessage(WM_ANALYZER_THREAD_SET_FORCE_CLOSE, 0, static_cast<LPARAM>(nButtonClose));
	}

	csLock.Unlock();		
}

void CStAnalyzerManager::HandleGraphSpecificFunc(std::auto_ptr<CStGIS_GraphData>& pData)
{
	CSingleLock cslock(&theApp.csgraph_);
	cslock.Lock();

	CStAnalyzerThread* pAnalyzerThread{ GetAnalyzerThread(pData->m_strTestName, pData->m_strTesterID) };

	if (pAnalyzerThread == nullptr)
		return;
	
	int nCommandID{ -1 };
	if (arrMapGisCommands.Lookup(pData->m_strCommandName.c_str(), nCommandID) == FALSE) {
		return;
	}

	////pAnalyzerThread->PostThreadMessage(UWM_ANALYZER_THREAD_CALL_GRAPH_FUNCTION, static_cast<WPARAM>(nCommandID), reinterpret_cast<LPARAM>(pData.release()));
	pAnalyzerThread->GetDialog()->PostMessage(UWM_CALL_GRAPH_FUNCTION, static_cast<WPARAM>(nCommandID), reinterpret_cast<LPARAM>(pData.release()));
}

void CStAnalyzerManager::HandleTextSpecificFunc(std::auto_ptr<CStGIS_TextData>& pData)
{
	CSingleLock cslock(&theApp.csgraph_);
	cslock.Lock();

	CStAnalyzerThread* pAnalyzerThread{ GetAnalyzerThread(pData->m_strTestName, pData->m_strTesterID) };

	if (pAnalyzerThread == nullptr)
		return;

	int nCommandID{ -1 };
	if (arrMapGisCommands.Lookup(pData->m_strCommandName.c_str(), nCommandID) == FALSE) {
		return;
	}

	////pAnalyzerThread->PostThreadMessage(UWM_ANALYZER_THREAD_PLOT_GIS_TEXT, static_cast<WPARAM>(nCommandID), reinterpret_cast<LPARAM>(pData.release()));
	pAnalyzerThread->GetDialog()->PostMessage(UWM_PLOT_GIS_TEXT, static_cast<WPARAM>(nCommandID), reinterpret_cast<LPARAM>(pData.release()));

}

void CStAnalyzerManager::HandleAnalyzerWindowMenu(const std::string& strTestName, const std::string& strTesterID)
{
	SetAnalyzerThreadDialogMenu(strTestName, strTesterID, 1);
}

void CStAnalyzerManager::RemoveAllAnalyzerThreads(void)
{
	bool bForceClose = true;

	for (auto itr = mapAnlyzerThreads_.begin(); itr != mapAnlyzerThreads_.end(); ++itr) {
		itr->second->PostThreadMessage(WM_ANALYZER_THREAD_SET_FORCE_CLOSE, 0, static_cast<LPARAM>(bForceClose));
		itr->second->PostThreadMessage(WM_QUIT, 0, 0);
		WaitForSingleObject(itr->second->m_hThread, INFINITE);
	}

	mapAnlyzerThreads_.clear();
}
